#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>

#include "ledpanel.h"

#define BUFFER 128

typedef enum { false, true } bool;

int readInt(char **str)
{
	char *s=(*str);
	int v=0;

	while (isdigit(*s))
		v=10*v + (*(s++))-'0';
	(*str)=s;
	return v;
}

bool getRange(char *str, uint16 *mask)
{
	uint16 m;
	char *old_str;
	int x1,x2;

	(*mask)=0;

	m=0;

	for (;;) {
		old_str=str;
		x1 = readInt(&str);
		if (old_str==str)
			return false;
		if ((*str)!='-') {
			m |= ((uint16)1<<x1);
		} else {
			str++;
			old_str=str;
			x2 = readInt(&str);
			if (old_str==str)
				return false;
			if (x1<=x2)
				m |= (((uint16)1<<(x2-x1+1))-1)<<x1;
		}
		if ((*str)!=',')
			break;
		str++;
	}

	(*mask) = m;
	return true;
}

struct {
	char name[16];
	int cmd;
} cmd_opt[] = {
	{ "on",     LED_ON },
	{ "off",    LED_OFF },
	{ "fast",   LED_FAST },
	{ "normal", LED_NORMAL },
	{ "slow",   LED_SLOW },
	{ "inter",  LED_INTER },
	{ "swap",   LED_SWAP },
	{ "",-1 }
};

int main(int argc, char *argv[])
{
	struct led_cmd lc;
	int msgid;
	int i;
	int p;
	uint16 r;

	if (argc<2) {
		printf("Usage: led [set|reset|quit|on range|off range]\n");
		exit(EXIT_FAILURE);
	}

	msgid = msgget((key_t)1234, 0666 | IPC_CREAT);

	if (msgid == -1) {
		perror("msgget");
		exit(EXIT_FAILURE);
	}

	lc.msg_type = 1;
	lc.state[LED_ON]=0;
	lc.state[LED_OFF]=0;
	lc.state[LED_FAST]=0;
	lc.state[LED_NORMAL]=0;
	lc.state[LED_SLOW]=0;
	lc.state[LED_INTER]=0;
	lc.state[LED_SWAP]=0;

	p=1;

	for (i=0;cmd_opt[i].cmd!=-1;i++) {
		if (!strcmp(argv[p],cmd_opt[i].name)) {
			lc.cmd = LED_CMD_SET;
			if (!getRange(argv[p+1], &r)) {
				fprintf(stderr, "Syntax error after %s\n", argv[p]);
				exit(EXIT_FAILURE);
			} else {
				lc.state[cmd_opt[i].cmd]|=r;
				p+=2;
				if (p>=argc)
					break;
			}
		}
	}
	if (p<argc) {
		if (!strcmp(argv[p], "reset")) {
			lc.cmd = LED_CMD_RESET;
		} else if (!strcmp(argv[p], "test")) {
			lc.cmd = LED_CMD_TEST;
		} else if (!strcmp(argv[p], "quit")) {
			lc.cmd = LED_CMD_QUIT;
		} else if (!strcmp(argv[p], "dump")) {
			lc.cmd = LED_CMD_DUMP;
		}
	}
	if (msgsnd(msgid, (void*)&lc, sizeof(lc), 0) == -1) {
		perror("msgsnd");
		exit(EXIT_FAILURE);
	}

	return 0;
}
