/*
 * LedPanel Deamon (ledd)
 *
 * version 0.0
 *
 */

/*
 * TODO: * ioperm mieux gere (en debut et fin de programme)
 *       - generer un clef IPC de facon correcte
 *       - remapper les leds (logique -> physique) (Par dfaut: 1-12 -> 0-11)
 *
 */

#define DEBUG_CODE

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/io.h>

#include "ledpanel.h"

#define LPT_PORT 0x378

/*
 * SLOW:   16/32
 * NORMAL:  8/16
 * FAST:    2/ 4
 * INTER:   3/16
 *
 */

int istate;
uint16 led_state[32];

void set_state(void)
{
	uint16 st;
	
	st=led_state[istate];
	outb((unsigned char)st,
	     (unsigned short)LPT_PORT);
	outb((unsigned char)((st>>8)^0x0B)&0x0F,
	     (unsigned short)LPT_PORT+2);
	
	istate=(istate+1)%32;
}

void cmd_test(void)
{
	int i;
	unsigned long state_test=0;
	
	for (i=0;i<12;i++) {
		state_test|=1L<<i;
		outb((unsigned char)state_test, (unsigned short)LPT_PORT);
		outb((unsigned char)((state_test>>8)^0x0B)&0x0F,
		     (unsigned short)LPT_PORT+2);
		usleep(100000);
	}
	for (i=0;i<12;i++) {
		state_test&=~(1L<<i);
		outb((unsigned char)state_test, (unsigned short)LPT_PORT);
		outb((unsigned char)((state_test>>8)^0x0B)&0x0F,
		     (unsigned short)LPT_PORT+2);
		usleep(100000);
	}
}

void change_state(int duration, int period, uint16 mask)
{
	int i,j;

	if (!mask)
		return;

	if (32%period) {
		printf("ERROR in change_state: (%d,%d)\n", duration, period);
		exit(EXIT_FAILURE);
	}

	i=0;
	do {
		for (j=0;j<period-duration;j++)
			led_state[i++] &= ~mask;
		for (j=0;j<duration;j++)
			led_state[i++] |= mask;
	} while (i<32);
}

int main_loop(void)
{
	struct led_cmd lc;
	int msgid;
	long msg_to_receive = 0;
	int i;
	
	msgid = msgget((key_t)1234, 0666 | IPC_CREAT);
	
	if (msgid == -1) {
		perror("msgget");
		exit(EXIT_FAILURE);
	}
	
	do {
		set_state();
		if (msgrcv(msgid, (void*)&lc, sizeof(lc),
			   msg_to_receive, IPC_NOWAIT) == -1) {
			if (errno != ENOMSG) {
				perror("msgrcv");
				msgctl(msgid, IPC_RMID, 0); // VERIFIER LE CODE DE RETOUR !!
				exit(EXIT_FAILURE);
			}
			usleep(80000);
			continue;
		}
		switch (lc.cmd) {
		case LED_CMD_DUMP:
			for (i=0;i<32;i++)
				printf("%04X\n", led_state[i]);
			break;
			
		case LED_CMD_RESET:
		case LED_CMD_QUIT:
			for (i=0;i<32;i++)
				led_state[i]=0;
			break;
		case LED_CMD_SET:
			change_state(32,32,lc.state[LED_ON]);
			change_state(0,32,lc.state[LED_OFF]);
			change_state(1,2,lc.state[LED_FAST]);
			change_state(4,8,lc.state[LED_NORMAL]);
			change_state(16,32,lc.state[LED_SLOW]);
			change_state(1,16,lc.state[LED_INTER]);
			
			if (lc.state[LED_SWAP])
				for (i=0;i<32;i++)
					led_state[i] ^= lc.state[LED_SWAP];
			break;
			
		case LED_CMD_TEST:
			cmd_test();
			break;
			
		default:
			printf("Unknown command %d\n", (int)lc.cmd);
		}
	} while (lc.cmd!=LED_CMD_QUIT);
	set_state();
	
	if (msgctl(msgid, IPC_RMID, 0) == -1) {
		perror("msgctl");
		exit(EXIT_FAILURE);
	}
	
	return 0;
}

int main(int argc, char *argv[])
{
	int ret_code;
	int i;

	for (i=1;i<argc;i++) {
		fprintf(stderr, "Usage: ledd\n");
		exit(EXIT_FAILURE);
	}
	
	//	printf("LedPanel Server v0.0\n\n");
	
	if (ioperm(LPT_PORT,3,1)==-1) {
		perror("ioperm");
		exit(EXIT_FAILURE);
	}
	
	cmd_test();
	
	ret_code = main_loop();
	
	if (ioperm(LPT_PORT,3,0)==-1) {
		perror("ioperm");
		exit(EXIT_FAILURE);
	}
	
	return ret_code;
}
